<?php

namespace helperComponent\helpers;

/**
 * Класс для работы с форматированием
 *
 * Class FormatHelper
 *
 * @package helperComponent\helpers
 */
class FormatHelper
{
    /**
     * Конвертировать значение в Boolean.
     * Вернет true, если $value равно 1, '1', true, 'true'
     * Вернет false, если $value равно 0, '0', false, 'false'
     *
     * @param $value - Конвертируемое значение
     *
     * @return bool
     * @throws \Exception Если $value не Boolean
     */
    public static function toBool($value)
    {
        // Если переменная уже boolean, вернуть без изменений
        if (is_bool($value)) {
            return $value;
        }

        $value = trim($value);

        switch ($value) {
            case 'true':
            case '1':
                return true;
            case 'false':
            case '0':
                return false;
        }
        throw new \Exception('invalid.boolean', 400);
    }

    /**
     * Преобразует первый символ строки в верхний регистр.
     * Отличается от стандартного тем, что можно указать кодировку, что позволяет работать с кирилицей
     *
     * @param string $string Исходная строка
     * @param string $encoding Кодировка
     *
     * @return string
     */
    public static function ucfirst(string $string, string $encoding = 'UTF-8'): string
    {
        return mb_strtoupper(mb_substr($string, 0, 1, $encoding), $encoding) . mb_substr($string, 1, null, $encoding);
    }

    /**
     * Приводит число к денежному формату
     *
     * @param float $amount Исходная сумма
     *
     * @return string
     */
    public static function currencyFormat(float $amount): string
    {
        return number_format($amount, 2, ',', ' ');
    }

    /**
     * Удаляет все лишние пробелы, в том числе и из середины строки
     *
     * @param string $string Исходная строка
     *
     * @return string
     */
    public static function trim(string $string): string
    {
        $string = trim($string);
        $string = str_replace('  ', ' ', $string);
        if (false !== stripos($string, '  ')) {
            $string = self::trim($string);
        }

        return $string;
    }

    /**
     * Автоматический пренос сторок
     *
     * @param string $string Исходная строка
     * @param int $rowLength Кол-во символов в одной строке
     *
     * @return string
     */
    public static function rowAutoWrap(?string $string, int $rowLength): ?string
    {
        $rows = [];
        if (mb_strlen($string, 'UTF-8') > $rowLength) {
            $rowNumber = 0;
            $words = explode(' ', $string);
            foreach ($words as $word) {
                $row = trim($rows[$rowNumber] . ' ' . $word);
                if (mb_strlen($row, 'UTF-8') > $rowLength) {
                    $rowNumber++;
                    $row = $word;
                }
                $rows[$rowNumber] = $row;
            }
        } else {
            return $string;
        }

        return implode(PHP_EOL, $rows);
    }
}
