<?php

namespace helperComponent\helpers;

use libphonenumber\Matcher;
use libphonenumber\PhoneNumberFormat;
use libphonenumber\PhoneNumberUtil;

/**
 * Класс для работы с телефонами
 *
 * Class PhoneHelper
 *
 * @package helperComponent\helpers
 */
class PhoneHelper
{
    /** Код страны Россия */
    const COUNTY_CODE_RU = 7;
    /** Регион Россия */
    const REGION_RU = 'RU';
    /** Регион неизвестен */
    const REGION_UNKNOWN = 'ZZ';

    /**
     * Форматирование по заданному формату номер телефона
     *
     * @param string $phoneNumber Номер телефона
     * @param int $format Формат телефона
     *
     * @return string Сформированный по заданному формату номер телефона
     */
    public function reformatPhoneNumber($phoneNumber, $format = PhoneNumberFormat::E164)
    {
        $phoneNumberUtil = PhoneNumberUtil::getInstance();
        $phoneNumberObject = $phoneNumberUtil->parse($phoneNumber, null);

        return $phoneNumberUtil->format($phoneNumberObject, $format);
    }

    /**
     * Проверяет передан российский номер телефона или нет
     *
     * @param string $phone Номер телефона
     *
     * @return bool
     */
    public static function isRussianPhone($phone)
    {
        $countryRegion = self::getCountryRegionByNumber($phone);
        if (in_array($countryRegion, [
            self::REGION_RU,
            self::REGION_UNKNOWN
        ])) {
            return true;
        }

        return false;
    }

    /**
     * Получить регион страны по номеру телефона
     *
     * @param string $phone Номер телефона
     *
     * @return string
     */
    public static function getCountryRegionByNumber(string $phone)
    {
        $phoneNumberUtil = PhoneNumberUtil::getInstance();
        // Проверяем валидность телефона и доробатывем если нужно
        $plusCharsPatternMatcher = new Matcher(PhoneNumberUtil::$PLUS_CHARS_PATTERN, $phone);
        if (!$plusCharsPatternMatcher->lookingAt()) {
            $phone = self::reformatRusPhone($phone);
            $phone = self::addPlusSingToPhone($phone);
        }
        try {
            $phoneNumberObject = $phoneNumberUtil->parse($phone, null);
            $countryCode = $phoneNumberObject->getCountryCode();
        } catch (\Exception $e) {
            $countryCode = self::COUNTY_CODE_RU;
        }

        return $phoneNumberUtil->getRegionCodeForCountryCode($countryCode);
    }

    /**
     * Реформация номера в нужный формат
     *
     * @param string $phone Номер телефона
     *
     * @return mixed
     */
    public static function reformatPhone(string $phone)
    {
        //Удаляем все лишнее из телефона кроме цифр
        $phone = preg_replace('~\D+~', '', $phone);
        //Если номер не русский то подмену не производим
        if (!self::isRussianPhone($phone)) {
            return $phone;
        }
        $phone = self::reformatRusPhone($phone);

        return $phone;
    }

    /**
     * Добавить в номер символ "+"
     *
     * @param string $phone Номер телефона
     *
     * @return string
     */
    public static function addPlusSingToPhone(string $phone)
    {
        //Удаляем все лишнее из телефона кроме цифр
        $phone = preg_replace('~\D+~', '', $phone);

        return PhoneNumberUtil::PLUS_SIGN . $phone;
    }

    /**
     * Реформация российского номера в нужный формат
     *
     * @param string $phone Номер телефона
     * @param int $search Символ для поиска в начале номера
     * @param int $replace Символ на замену в начале номера
     *
     * @return mixed
     */
    public static function reformatRusPhone(string $phone, $search = 8, $replace = 7)
    {
        //Удаляем все лишнее из телефона кроме цифр
        $phone = preg_replace('~\D+~', '', $phone);
        //Заменяем первый символ номера с искомого на нужный
        if (substr($phone, 0, 1) == $search && strlen($phone) == 11) {
            $phone[0] = $replace;
        }

        return $phone;
    }
}
